@@ -2,6 +2,8 @@ package org.gcube.portlets.user.gisviewer.client;
 
 
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.Date;
 import java.util.List;
 
@@ -9,6 +11,7 @@ import org.gcube.common.core.utils.logging.GCUBELog;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.DataResult;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.GeoserverItem;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.GroupInfo;
+import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerConfiguration;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItemsResult;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.MapViewInfo;
@@ -166,7 +169,7 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 
 				if (result.getLayerItemsSize()!=0) {
 					List<LayerItem> layerItems = result.getLayerItems();
-					addLayers(layerItems, false);
+					addLayersOnMap(layerItems, false);
 					showLayer(layerItems.get(layerItems.size()-1)); // show last layer
 				}
 			}
@@ -178,36 +181,11 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 		});
 	}
 
-//	public void addGroupByGroupName(final String groupName) {
-//		waitMessage(true);
-//		gisViewerServiceAsync.getGroupsInfo(groupName, new AsyncCallback<GroupInfo>() {
-//			public void onSuccess(GroupInfo groupInfo) {
-//				waitMessage(false);
-//				if (groupInfo==null) {
-//					MessageBox.alert("Error", "Group \""+groupName+"\" not found.", null);
-//					return;
-//				}
-//				if (groupInfo.getBounds()==null) {
-//					MessageBox.alert("Error", "Group Bounds is null", null);
-//					return;
-//				}
-//
-//				//				if (projection==null)
-//				//					projection = groupInfo.getBounds().getCrs();
-//				//
-//				//addGroupByName2(groupInfo, Constants.defaultGroup2);
-//				addLayers(groupInfo.getLayers(), false);				
-//			}
-//
-//			public void onFailure(Throwable caught) {
-//				waitMessage(false);
-//				caught.printStackTrace();
-//				Window.alert("The Group " + groupName + " doesn't exist");
-//			}
-//		});
-//	}
+	public void addLayersByLayerNames(String ... layerNames) {
+		 addLayersByLayerNames(Arrays.asList(layerNames));
+	}
 
-	public void addLayersByLayerNames(final List<String> layerNames) {
+	public void addLayersByLayerNames(List<String> layerNames) {
 		addLayersByLayerNames(layerNames, false);
 	}
 
@@ -215,28 +193,6 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 		addLayersByLayerNames(layerNames, true);
 	}
 
-	private void addLayersByLayerNames(final List<String> layerNames, final boolean toTop) {
-		waitMessage(true);
-		gisViewerServiceAsync.getLayersInfo(layerNames, new AsyncCallback<LayerItemsResult>() {			
-			public void onSuccess(LayerItemsResult result) {
-				waitMessage(false);
-				if (!result.getStatus().equals(""))
-					MessageBox.alert("Error", result.getStatus(), null);
-
-				//				if (projection==null)
-				//					projection = Constants.defaultProjection;
-				//				
-				if (result.getLayerItemsSize()!=0)
-					addLayers(result.getLayerItems(), toTop);
-			}
-
-			public void onFailure(Throwable caught) {
-				waitMessage(false);
-				Window.alert("The layers doesn't exist");
-			}
-		});
-	}
-
 	/**
 	 * @param layerItems
 	 */
@@ -269,6 +225,73 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 		addLayersByLayerItems(layerItems, true);
 	}
 
+	public void addLayersByLayerConfigurations(LayerConfiguration ... layersConfigurations) {
+		this.addLayersByLayerConfigurations(Arrays.asList(layersConfigurations), false);
+	}
+
+	public void addLayersByLayerConfigurationsToTop(LayerConfiguration ... layersConfigurations) {
+		this.addLayersByLayerConfigurations(Arrays.asList(layersConfigurations), true);
+	}
+
+	private void addLayersByLayerConfigurations(final List<LayerConfiguration> layerConfigurations, final boolean toTop) {
+		waitMessage(true);
+		gisViewerServiceAsync.getLayersInfoByLayerConfigurations(layerConfigurations, new AsyncCallback<LayerItemsResult>() {
+			
+			public void onSuccess(LayerItemsResult result) {
+				waitMessage(false);
+				if (!result.getStatus().equals(""))
+					MessageBox.alert("Error", result.getStatus(), null);
+
+				if (result.getLayerItemsSize()!=0)
+					addLayersOnMap(result.getLayerItems(), toTop);
+			}
+
+			public void onFailure(Throwable caught) {
+				waitMessage(false);
+				caught.printStackTrace();
+				Window.alert("The layers doesn't exist");
+			}
+		});
+	}
+	
+	private void addLayersByLayerNames(final List<String> layerNames, final boolean toTop) {
+		waitMessage(true);
+		gisViewerServiceAsync.getLayersInfo(layerNames, new AsyncCallback<LayerItemsResult>() {			
+			public void onSuccess(LayerItemsResult result) {
+				waitMessage(false);
+				if (!result.getStatus().equals(""))
+					MessageBox.alert("Error", result.getStatus(), null);
+
+				if (result.getLayerItemsSize()!=0)
+					addLayersOnMap(result.getLayerItems(), toTop);
+			}
+
+			public void onFailure(Throwable caught) {
+				waitMessage(false);
+				Window.alert("The layers doesn't exist");
+			}
+		});
+	}
+
+	private void addLayersByLayerItems(List<LayerItem> layerItems, final boolean onTop) {
+		waitMessage(true);
+		gisViewerServiceAsync.getLayersInfoByLayerItems(layerItems, new AsyncCallback<LayerItemsResult>() {			
+			public void onSuccess(LayerItemsResult result) {
+				waitMessage(false);
+				if (!result.getStatus().equals(""))
+					MessageBox.alert("Error", result.getStatus(), null);
+
+				if (result.getLayerItemsSize()!=0)
+					addLayersOnMap(result.getLayerItems(), onTop);
+			}
+
+			public void onFailure(Throwable caught) {
+				waitMessage(false);
+				Window.alert("The layers doesn't exist");
+			}
+		});
+	}
+	
 	public void addLayerByWms(String title, String layerName, String url, boolean isBase, boolean displayInLayerSwitcher) {
 		LayerItem l = new LayerItem();
 		l.setBaseLayer(isBase);
@@ -288,29 +311,7 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 		layersPanel.updateLayersOrder();
 	}
 
-	private void addLayersByLayerItems(List<LayerItem> layerItems, final boolean onTop) {
-		waitMessage(true);
-		gisViewerServiceAsync.getLayersInfoByLayerItems(layerItems, new AsyncCallback<LayerItemsResult>() {			
-			public void onSuccess(LayerItemsResult result) {
-				waitMessage(false);
-				if (!result.getStatus().equals(""))
-					MessageBox.alert("Error", result.getStatus(), null);
-
-				//				if (projection==null)
-				//					projection = Constants.defaultProjection;
-				//				
-				if (result.getLayerItemsSize()!=0)
-					addLayers(result.getLayerItems(), onTop);
-			}
-
-			public void onFailure(Throwable caught) {
-				waitMessage(false);
-				Window.alert("The layers doesn't exist");
-			}
-		});
-	}
-	
-	private void addLayers(List<LayerItem> layerItems, boolean toTop) {
+	private void addLayersOnMap(List<LayerItem> layerItems, boolean toTop) {
 		for (LayerItem layerItem : layerItems)
 			layerItem.setBaseLayer(false);
 		openLayersMap.addLayerItems(layerItems, toTop);
@@ -483,19 +484,57 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 
 
 	public void saveMapImage(String outputFormat, boolean isWMS) {
-		String url = getMapImageUrl(outputFormat, isWMS);
+		//String url = getMapImageUrl(outputFormat, isWMS);
 		
 		String fileName = "MapImage_"+DateTimeFormat.getFormat("yyyy-MM-dd").format(new Date());
 		
-//		Date date = new Date();
-//		int Month = date.getMonth();
-//		int Day = date.getDate();
-//		int Year = date.getYear();
-//		int Hour = date.getHours();
-//		int min = date.getMinutes();
-//		int sec = date.getSeconds();
+		MapViewInfo mapViewInfo = openLayersMap.getMapViewInfo();
 		
-		gisViewerSaveHandler.saveMapImage(fileName, outputFormat, url);
+		String bbox = mapViewInfo.getLowerLeftX()
+				+ "," + mapViewInfo.getLowerLeftY()
+				+ "," + mapViewInfo.getUpperRightX()
+				+ "," + mapViewInfo.getUpperRightY();
+		String width = ""+mapViewInfo.getWidth();
+		String height = ""+mapViewInfo.getHeight();
+
+		// get visible layers
+		List<LayerItem> visibleLayers = new ArrayList<LayerItem>();
+		visibleLayers.add(openLayersMap.getActiveBaseLayer());
+		visibleLayers.addAll(layersPanel.getVisibleLayers());
+		List<GeoserverItem> geoserverItems = groupLayersByGeoserver(visibleLayers, false);
+
+		ArrayList<String> geoservers = new ArrayList<String>();
+		ArrayList<String> layers = new ArrayList<String>();
+		ArrayList<String> styles = new ArrayList<String>();
+		ArrayList<String> opacities = new ArrayList<String>();
+		ArrayList<String> cqlfilters = new ArrayList<String>();
+		ArrayList<String> gsrefs = new ArrayList<String>();
+		
+
+		int gsIndex=0;
+		for (GeoserverItem geoserverItem: geoserverItems) {
+			geoservers.add(geoserverItem.getUrl());
+			//geoservers.add(URLMakers.encodeUrl(geoserverItem.getUrl()));
+
+			for (LayerItem layerItem: geoserverItem.getLayerItems()) {
+				String cqlFilter = layerItem.getCqlFilter();
+				
+				layers.add(layerItem.getLayer());
+				styles.add((layerItem.getStyle()==null ? "null" : layerItem.getStyle()));
+				opacities.add(""+layerItem.getOpacity());
+				cqlfilters.add((cqlFilter.equals("") ? "null" : cqlFilter));
+
+				gsrefs.add(""+gsIndex);
+			}
+		}
+
+		gisViewerSaveHandler.saveMapImage(fileName, outputFormat, bbox, width, height, 
+				geoservers.toArray(new String[geoservers.size()]), 
+				layers.toArray(new String[layers.size()]), 
+				styles.toArray(new String[styles.size()]), 
+				opacities.toArray(new String[opacities.size()]), 
+				cqlfilters.toArray(new String[cqlfilters.size()]), 
+				gsrefs.toArray(new String[gsrefs.size()]));
 	}
 
 	private String getMapImageUrl(String outputFormat, boolean isWMS) {
@@ -755,4 +794,5 @@ implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandle
 			showDataPanel();
 		}
 	}
+
 }
