@@ -5,6 +5,7 @@ import java.util.List;
 
 import org.gcube.portlets.user.gisviewer.client.commons.beans.DataResult;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.GroupInfo;
+import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerConfiguration;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItemsResult;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.SavedGroup;
@@ -23,10 +24,12 @@ public interface GisViewerServiceAsync {
 
 	public void getLayersInfo(List<String> layersName, AsyncCallback<LayerItemsResult> callback);
 
-	void getTransectUrl(AsyncCallback<String> callback);
-
 	public void getLayersInfoByLayerItems(List<LayerItem> layerItems, AsyncCallback<LayerItemsResult> asyncCallback);
 
+	public void getLayersInfoByLayerConfigurations(List<LayerConfiguration> layerConfigurations, AsyncCallback<LayerItemsResult> asyncCallback);
+
+	void getTransectUrl(AsyncCallback<String> callback);
+
 	public void getDataResult(List<LayerItem> layerItems, String bbox, AsyncCallback<List<WfsTable>> asyncCallback);
 
 	//public void saveLayersGroup(GroupInfo groupInfo, String format, AsyncCallback<SavedGroup> callback);
